package de.uni_mannheim.informatik.dws.semtec;

import java.io.IOException;
import org.apache.jena.rdf.model.Literal;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.RDFNode;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;
import org.apache.jena.rdf.model.Statement;
import org.apache.jena.rdf.model.StmtIterator;
import org.apache.jena.riot.Lang;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.vocabulary.RDFS;

public class JenaNavigation {

    public static void main(String[] args) throws IOException {
        //Create an RDF Graph
        Model model = ModelFactory.createDefaultModel();
        //load data from URL into graph
        //URI of Kaiserslautern is http://dbpedia.org/resource/Mannheim
        //when query with accept header "application/rdf+xml" then this URL redirects to http://dbpedia.org/data/Mannheim
        //have a detailed look at
        //wget --header="Accept: application/rdf+xml" http://dbpedia.org/resource/Mannheim
        //therefor read data directly from "http://dbpedia.org/data/Mannheim" because Jena does not follow redirects
        model.read("http://dbpedia.org/data/Mannheim");

        //Write the whole RDF graph to standard out formatted as turtle
        RDFDataMgr.write(System.out, model, Lang.TURTLE);        

        //retrieve resource mannheim from the graph
        Resource mannheim = model.getResource("http://dbpedia.org/resource/Mannheim");
        //retrive property from model and follow that property given mannheim as subject. It only returns the first object available
        Resource countryOfMannheim = mannheim.getProperty(model.getProperty("http://dbpedia.org/ontology/country")).getResource();

        System.out.println("\n\nSystem.out.println(countryOfMannheim);");
        System.out.println(countryOfMannheim);

        
        //Instead of 
        //Literal lit = mannheim.getProperty(ResourceFactory.createProperty("http://www.w3.org/2000/01/rdf-schema#label")).getLiteral();
        //you could also write the following because some properties and resources are already defined (like RDFS.label).
        //sidenote: this also retrieves only the first available literal
        Literal lit = mannheim.getProperty(RDFS.label).getLiteral();
        System.out.println("lit.getString() -> " + lit.getString());
        System.out.println("lit.getLanguage()  -> " + lit.getLanguage());
        System.out.println("lit.getDatatype() -> " + lit.getDatatype());

        //iterate over all available labels of resource kaiserslautern:
        StmtIterator iter = mannheim.listProperties(RDFS.label);
        while (iter.hasNext()) {
            Statement s = iter.next();
            RDFNode node = s.getObject();
            //check if object is really a literal
            if (node.isLiteral()) {
                System.out.println("node.asLiteral().getString() -> " + node.asLiteral().getString());
            }
        }
        //basic hierarchy of Jena:
        //        RDFNode
        //        ^     ^ 
        //        |     | 
        //    Literal Resource
    }
}
